@@ -2,6 +2,35 @@ use crate::{error::*, value::intermediate::Intermediate};
 use petgraph::{algo::astar, Graph};
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum DiffOptimizationHint {
+    Default,
+    SizeSource,
+    SizeTarget,
+    /// (bytesize threshold)
+    SizeValue(usize),
+    /// (percentage threshold)
+    SizePercentage(f64),
+}
+
+impl Default for DiffOptimizationHint {
+    fn default() -> Self {
+        Self::Default
+    }
+}
+
+#[derive(Debug, Default, Copy, Clone, PartialEq)]
+pub struct DiffOptions {
+    pub optimization_hint: DiffOptimizationHint,
+}
+
+impl DiffOptions {
+    pub fn optimization_hint(mut self, hint: DiffOptimizationHint) -> Self {
+        self.optimization_hint = hint;
+        self
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, PartialOrd, Serialize, Deserialize)]
 pub enum Change {
     Same,
@@ -88,31 +117,74 @@ impl Change {
         matches!(self, Self::Same)
     }
 
-    pub fn difference(prev: &Intermediate, next: &Intermediate) -> Self {
+    fn optimize(
+        self,
+        source: &Intermediate,
+        target: &Intermediate,
+        hint: DiffOptimizationHint,
+    ) -> Self {
+        match hint {
+            DiffOptimizationHint::Default => self,
+            DiffOptimizationHint::SizeSource => {
+                if self.total_bytesize() > source.total_bytesize() {
+                    Self::Changed(target.to_owned())
+                } else {
+                    self
+                }
+            }
+            DiffOptimizationHint::SizeTarget => {
+                if self.total_bytesize() > target.total_bytesize() {
+                    Self::Changed(target.to_owned())
+                } else {
+                    self
+                }
+            }
+            DiffOptimizationHint::SizeValue(threshold) => {
+                if self.total_bytesize() > threshold {
+                    Self::Changed(target.to_owned())
+                } else {
+                    self
+                }
+            }
+            DiffOptimizationHint::SizePercentage(threshold) => {
+                if self.total_bytesize()
+                    > (threshold.max(0.0).min(1.0) * source.total_bytesize() as f64) as _
+                {
+                    Self::Changed(target.to_owned())
+                } else {
+                    self
+                }
+            }
+        }
+    }
+
+    pub fn difference(prev: &Intermediate, next: &Intermediate, options: &DiffOptions) -> Self {
         if prev == next {
             Self::Same
         } else {
             match (prev, next) {
                 (Intermediate::Option(Some(prev)), Intermediate::Option(Some(next))) => {
-                    Self::PartialChange(Box::new(Self::difference(prev, next)))
+                    Self::PartialChange(Box::new(Self::difference(prev, next, options)))
                 }
                 (Intermediate::NewTypeStruct(prev), Intermediate::NewTypeStruct(next)) => {
-                    Self::PartialChange(Box::new(Self::difference(prev, next)))
+                    Self::PartialChange(Box::new(Self::difference(prev, next, options)))
                 }
                 (
-                    Intermediate::NewTypeVariant(prev_name, prev_index, prev_value),
-                    Intermediate::NewTypeVariant(next_name, next_index, next_value),
+                    Intermediate::NewTypeVariant(prev_name, prev_value),
+                    Intermediate::NewTypeVariant(next_name, next_value),
                 ) => {
-                    if prev_name != next_name || prev_index != next_index {
+                    if prev_name != next_name {
                         Self::Changed(next.to_owned())
                     } else {
-                        Self::PartialChange(Box::new(Self::difference(prev_value, next_value)))
+                        Self::PartialChange(Box::new(Self::difference(
+                            prev_value, next_value, options,
+                        )))
                     }
                 }
                 (Intermediate::Seq(prev), Intermediate::Seq(next))
                 | (Intermediate::Tuple(prev), Intermediate::Tuple(next))
                 | (Intermediate::TupleStruct(prev), Intermediate::TupleStruct(next)) => {
-                    Self::PartialSeq(Self::sequence_difference(prev, next))
+                    Self::PartialSeq(Self::sequence_difference(prev, next, options))
                 }
                 (Intermediate::Map(prev), Intermediate::Map(next)) => {
                     let mut result = vec![];
@@ -132,7 +204,7 @@ impl Change {
                             .find(|(nk, _)| pk == nk)
                             .filter(|(_, nv)| pv != nv)
                         {
-                            let diff = Self::difference(pv, nv);
+                            let diff = Self::difference(pv, nv, options);
                             if !diff.is_same() {
                                 result.push((pk.to_owned(), diff));
                             }
@@ -141,10 +213,7 @@ impl Change {
                     Self::PartialMap(result)
                 }
                 (Intermediate::Struct(prev), Intermediate::Struct(next))
-                | (
-                    Intermediate::StructVariant(_, _, prev),
-                    Intermediate::StructVariant(_, _, next),
-                ) => {
+                | (Intermediate::StructVariant(_, prev), Intermediate::StructVariant(_, next)) => {
                     let mut result = vec![];
                     for (nk, nv) in next {
                         if !prev.iter().any(|(pk, _)| pk == nk) {
@@ -162,7 +231,7 @@ impl Change {
                             .find(|(nk, _)| pk == nk)
                             .filter(|(_, nv)| pv != nv)
                         {
-                            let diff = Self::difference(pv, nv);
+                            let diff = Self::difference(pv, nv, options);
                             if !diff.is_same() {
                                 result.push((pk.to_owned(), diff));
                             }
@@ -173,9 +242,14 @@ impl Change {
                 _ => Self::Changed(next.to_owned()),
             }
         }
+        .optimize(prev, next, options.optimization_hint)
     }
 
-    pub fn sequence_difference(prev: &[Intermediate], next: &[Intermediate]) -> Vec<(usize, Self)> {
+    pub fn sequence_difference(
+        prev: &[Intermediate],
+        next: &[Intermediate],
+        options: &DiffOptions,
+    ) -> Vec<(usize, Self)> {
         if prev.is_empty() && next.is_empty() {
             return vec![];
         } else if prev.is_empty() {
@@ -273,7 +347,15 @@ impl Change {
                         }
                         Diff::Changed => {
                             pos += 1;
-                            Some((old_pos, Self::Changed(next[old_pos].to_owned())))
+                            let (prev_pos, _) = graph.node_weight(chunk[0])?;
+                            let prev = &prev[*prev_pos];
+                            let next = &next[old_pos];
+                            let diff = Self::difference(prev, next, options).optimize(
+                                prev,
+                                next,
+                                options.optimization_hint,
+                            );
+                            Some((old_pos, diff))
                         }
                         Diff::Removed => Some((old_pos, Self::Removed)),
                         Diff::Added => {
@@ -293,7 +375,24 @@ impl Change {
             Self::Removed => Ok(None),
             Self::Changed(v) => Ok(Some(v.to_owned())),
             Self::Added(_) => Err(Error::CannotAdd(value.to_owned())),
-            Self::PartialChange(change) => change.patch(value),
+            Self::PartialChange(change) => {
+                let result = match value {
+                    Intermediate::Option(Some(v)) => match change.patch(v)? {
+                        Some(v) => Intermediate::Option(Some(Box::new(v))),
+                        _ => return Err(Error::NotPartial(value.to_owned())),
+                    },
+                    Intermediate::NewTypeStruct(v) => match change.patch(v)? {
+                        Some(v) => Intermediate::NewTypeStruct(Box::new(v)),
+                        _ => return Err(Error::NotPartial(value.to_owned())),
+                    },
+                    Intermediate::NewTypeVariant(n, v) => match change.patch(v)? {
+                        Some(v) => Intermediate::NewTypeVariant(n.to_owned(), Box::new(v)),
+                        _ => return Err(Error::NotPartial(value.to_owned())),
+                    },
+                    _ => return Err(Error::NotPartial(value.to_owned())),
+                };
+                Ok(Some(result))
+            }
             Self::PartialSeq(changes) => {
                 fn implement(
                     v: &[Intermediate],
@@ -412,9 +511,8 @@ impl Change {
                     Intermediate::Struct(v) => {
                         Ok(Some(Intermediate::Struct(implement(v, changes)?)))
                     }
-                    Intermediate::StructVariant(n, i, v) => Ok(Some(Intermediate::StructVariant(
+                    Intermediate::StructVariant(n, v) => Ok(Some(Intermediate::StructVariant(
                         n.to_owned(),
-                        *i,
                         implement(v, changes)?,
                     ))),
                     _ => Err(Error::NotMap(value.to_owned())),
@@ -423,14 +521,14 @@ impl Change {
         }
     }
 
-    pub fn data_difference<P, N>(prev: &P, next: &N) -> Result<Self>
+    pub fn data_difference<P, N>(prev: &P, next: &N, options: &DiffOptions) -> Result<Self>
     where
         P: Serialize,
         N: Serialize,
     {
         let prev = crate::to_intermediate(prev)?;
         let next = crate::to_intermediate(next)?;
-        Ok(Self::difference(&prev, &next))
+        Ok(Self::difference(&prev, &next, options))
     }
 
     pub fn data_patch<T>(&self, data: &T) -> Result<Option<T>>
